//=============================================================================
// Stahl Plugin - State Tiering
// Stahl_StateTier.js    VERSION 1.1.1
//=============================================================================

var Imported = Imported || {};
Imported.Stahl_StateTier = true;

var Stahl = Stahl || {};
Stahl.StateTier = Stahl.StateTier || {};

//=============================================================================
 /*:
 * @plugindesc State tiering for OMORI
 * @author ReynStahl
 * @help
 * ======== EXAMPLE FUNCTIONS ======== (put in eval for example)
 * Stahl.StateTier.AddStateTier(type, tier, target)
 * 
 * type: (String) "sad", "angry", "happy", "atk", "def", "spd", 
 * "randEmo" - Random Emotion
 * "randBuff" - Random Buffs
 * "supBuff" - Supplementary buffs, Buff according to emotion.
 * tier: (Int) amount of tier addded on
 * target: Battler
 * 
 * ======== EXAMPLE VARIABLE CALLS ========
 * stateTypeTier(type)
 * type: (String) "sad", "angry", "happy", "atk", "def", "spd"
 * $gameActors.actor(17).stateTypeTier("atk")
 * returns the tier of attack states of actor 17
 */
 //=============================================================================

(function($) {
	// ################################################################
	//                         LOCAL FUNCTIONS
	// ################################################################

	//Array of State IDs from low to high. 0 is No state.
	function stateArray(type){
		switch(type.toLowerCase()) {
			case "sad": //EMOTIONS
				return [0,10,11,12];
			case "angry":
				return [0,14,15,16];
			case "happy":
				return [0,6,7,8];
			case "atk": case "attack": //BUFF TYPES
				return [94,93,92,0,89,90,91];
			case "def": case "defense":
				return [100,99,98,0,95,96,97];
			case "spd": case "agi": case "speed": case "agility":
				return [106,105,104,0,101,102,103];
			//You can add own list too!
			//case "hit": case "hitrate":
			//	return [233,232,231,0,228,229,230];
			default:
				return [0];
		}
	}

	// ################################################################
	//                    STATE ADDING AND REMOVING
	// ################################################################
	
	// State Tiering
	$.AddStateTier = function(type, tier, target) {
		//if 0 tier then return, it won't do anything lol
		if (tier == 0) {
			return;
		}

		//random emotion and buff generator
		var _type = type.toLowerCase();
		switch(_type) {
			case "randemo":
				_type = ["happy", "sad", "angry"][Math.randomInt(3)];
				break;
			case "randbuff":
				_type = ["atk", "def", "spd"][Math.randomInt(4)];
				break;
			case "supbuff":
				if (stateArray("happy").some(target.isStateAffected.bind(target))){_type = "spd";}
				else if (stateArray("sad").some(target.isStateAffected.bind(target))){_type = "def";}
				else if (stateArray("angry").some(target.isStateAffected.bind(target))){_type = "atk";}
				else {return;}
				break;
		}

		var arr = stateArray(_type); //Array of State IDs from low to high. 0 is No state.
		var currentState = 0; //current state ID
		var finalState = 0; //final state ID

		//find current state. Tried turning this into function but it doesn't really work lol
		for (const a of arr) {
			if (a !== 0 && target.isStateAffected(a)) {
				currentState = a;
				break;
			}
  		}

		//add tier by checking index. Clamp to min max
		const MIN = 0;
		const MAX = arr.length - 1;
		const currentStateIndex = arr.indexOf(currentState); //so it doesnt try to call array multiple times
		finalState = arr[Math.min(Math.max(currentStateIndex + tier, MIN), MAX)];

		//Check resistance, is it goes to 0 then all is immune, therefore nothing happens
		let resistCheck = tier;
		while (target.isStateResist(finalState) && resistCheck !== 0) {
			//check "lesser" state. If +3 but is immune, then try +2 (positive then lower); If -3 but is immune, then try -2 (negative then higher); etc.
			resistCheck += resistCheck > 0 ? -1 : 1;
			finalState = arr[Math.min(Math.max(currentStateIndex + resistCheck, MIN), MAX)];
		}

		//remove old state
		if (finalState != currentState && currentState !== 0)
			target.removeState(currentState);

		//add new state if its not 0, which is no state
		if (finalState != 0)
			target.addState(finalState);

		SceneManager._scene._statusWindow.refresh();
	}

	// ################################################################
	//                       VARIABLE CALLS
	// ################################################################

	//returns the "tier" of the state.
	Game_BattlerBase.prototype.stateTypeTier = function(type) {
		var arr = stateArray(type); 
		var midStateIndex = arr.indexOf[0];
		var currentState = 0;

		//find current state, found nothing then is 0;
		for (const a of arr) {
			if (a !== 0 && this.isStateAffected(a)) {
				currentState = a;
				break;
			}
  		}

		const currentStateIndex = arr.indexOf(currentState);
		//tier is difference between the middle no-state to current state
		return currentStateIndex - midStateIndex;
	}
})(Stahl.StateTier);